# -*- coding: UTF-8 -*-
# (c)2013 Mik Kocikowski, MIT License (http://opensource.org/licenses/MIT)
# https://github.com/mkocikowski/esbench

import datetime
import os.path
import unittest
import json
import logging

import esbench.analyze
import esbench.api
import esbench.test.test_api

logging.basicConfig(level=logging.DEBUG)


class AnalyzeTest(unittest.TestCase):

    def test_get_benchmarks(self):
        conn = esbench.api.Conn(conn_cls=esbench.test.test_api.MockHTTPConnection)
        resp = esbench.analyze._get_benchmarks(conn)
        self.assertEqual(resp.curl, 'curl -XGET http://localhost:9200/%s/bench/_search?sort=benchmark_start:asc&size=100' % esbench.STATS_INDEX_NAME)


    def test_benchmarks(self):
        resp = esbench.api.ApiResponse(status=200, reason='OK', data='{"took":1,"timed_out":false,"_shards":{"total":5,"successful":5,"failed":0},"hits":{"total":2,"max_score":null,"hits":[{"_index":"stats","_type":"bench","_id":"378b42da","_score":null, "_source" : {"argv": {"append": false, "command": "run", "config_file_path": "/Users/z013rqg/dev/esbench/esbench/config.json", "data": null, "n": 500000, "name": "t1", "no_load": false, "no_optimize_calls": false, "observations": 20, "repetitions": 500, "segments": null, "verbose": false}, "benchmark_id": "378b42da", "benchmark_name": "t1", "benchmark_start": "2013-11-22T19:01:47Z", "benchmark_stop": "2013-11-22T21:09:00Z", "cmnd": "run --name t1 --repetitions 500 --observations 20 500000", "config": "{\\"index\\": {\\"mappings\\": {\\"doc\\": {\\"_size\\": {\\"enabled\\": true, \\"store\\": \\"yes\\"}, \\"_source\\": {\\"enabled\\": true}, \\"properties\\": {\\"abstract\\": {\\"properties\\": {\\"txt\\": {\\"store\\": \\"yes\\", \\"type\\": \\"string\\"}}}}}}, \\"settings\\": {\\"index\\": {\\"number_of_replicas\\": 0, \\"number_of_shards\\": 1}}}, \\"name_doctype\\": \\"doc\\", \\"name_index\\": \\"test\\", \\"queries\\": {\\"match\\": {\\"fields\\": [\\"description.txt\\"], \\"from\\": 0, \\"query\\": {\\"match\\": {\\"description.txt\\": \\"computing V%(variable)s device portable\\"}}, \\"size\\": 10}, \\"match_abs\\": {\\"fields\\": [\\"abstract.txt\\"], \\"from\\": 0, \\"query\\": {\\"match\\": {\\"description.txt\\": \\"computing V%(variable)s device portable\\"}}, \\"size\\": 10}, \\"match_no_rnd\\": {\\"fields\\": [\\"description.txt\\"], \\"from\\": 0, \\"query\\": {\\"match\\": {\\"description.txt\\": \\"computing device portable\\"}}, \\"size\\": 10}, \\"match_srt\\": {\\"fields\\": [\\"description.txt\\"], \\"from\\": 0, \\"query\\": {\\"match\\": {\\"description.txt\\": \\"computing V%(variable)s device portable\\"}}, \\"size\\": 10, \\"sort\\": [{\\"_meta.date_published\\": \\"desc\\"}]}, \\"mlt\\": {\\"fields\\": [\\"description.txt\\"], \\"from\\": 0, \\"query\\": {\\"more_like_this\\": {\\"fields\\": [\\"description.txt\\"], \\"like_text\\": \\"USING V%(variable)s PROTECTING THE CARTRIDGE FROM WELDING HEAT\\", \\"max_query_terms\\": 25, \\"min_term_freq\\": 2, \\"percent_terms_to_match\\": 0.3}}, \\"size\\": 10}}}", "t_total": "127.22m", "t_total_in_millis": 7632910},"sort":[1385146907000]},{"_index":"stats","_type":"bench","_id":"315fd1aa","_score":null, "_source" : {"argv": {"append": false, "command": "run", "config_file_path": "/Users/z013rqg/dev/esbench/esbench/config.json", "data": null, "n": 100, "name": "14109fd2699b.local::2013-11-26T17:58:10Z", "no_load": false, "no_optimize_calls": false, "observations": 10, "repetitions": 100, "segments": null, "verbose": false}, "benchmark_id": "315fd1aa", "benchmark_name": "14109fd2699b.local::2013-11-26T17:58:10Z", "benchmark_start": "2013-11-26T17:58:10Z", "benchmark_stop": "2013-11-26T17:58:36Z", "cmnd": "run", "config": "{\\"index\\": {\\"mappings\\": {\\"doc\\": {\\"_size\\": {\\"enabled\\": true, \\"store\\": \\"yes\\"}, \\"_source\\": {\\"enabled\\": true}, \\"properties\\": {\\"abstract\\": {\\"properties\\": {\\"txt\\": {\\"store\\": \\"yes\\", \\"type\\": \\"string\\"}}}}}}, \\"settings\\": {\\"index\\": {\\"number_of_replicas\\": 0, \\"number_of_shards\\": 1}}}, \\"name_doctype\\": \\"doc\\", \\"name_index\\": \\"test\\", \\"queries\\": {\\"match\\": {\\"fields\\": [\\"description.txt\\"], \\"from\\": 0, \\"query\\": {\\"match\\": {\\"description.txt\\": \\"computing V%(variable)s device portable\\"}}, \\"size\\": 10}, \\"match_abs\\": {\\"fields\\": [\\"abstract.txt\\"], \\"from\\": 0, \\"query\\": {\\"match\\": {\\"description.txt\\": \\"computing V%(variable)s device portable\\"}}, \\"size\\": 10}, \\"match_no_rnd\\": {\\"fields\\": [\\"description.txt\\"], \\"from\\": 0, \\"query\\": {\\"match\\": {\\"description.txt\\": \\"computing device portable\\"}}, \\"size\\": 10}, \\"match_srt\\": {\\"fields\\": [\\"description.txt\\"], \\"from\\": 0, \\"query\\": {\\"match\\": {\\"description.txt\\": \\"computing V%(variable)s device portable\\"}}, \\"size\\": 10, \\"sort\\": [{\\"_meta.date_published\\": \\"desc\\"}]}, \\"mlt\\": {\\"fields\\": [\\"description.txt\\"], \\"from\\": 0, \\"query\\": {\\"more_like_this\\": {\\"fields\\": [\\"description.txt\\"], \\"like_text\\": \\"USING V%(variable)s PROTECTING THE CARTRIDGE FROM WELDING HEAT\\", \\"max_query_terms\\": 25, \\"min_term_freq\\": 2, \\"percent_terms_to_match\\": 0.3}}, \\"size\\": 10}}}", "t_total": "0.43m", "t_total_in_millis": 25744},"sort":[1385488690000]}]}}', curl='curl -XGET http://localhost:9200/stats/bench/_search?sort=benchmark_start:asc&size=100')
        self.assertEqual(2, len(list(esbench.analyze.benchmarks(resp=resp))))
        self.assertEqual(1, len(list(esbench.analyze.benchmarks(resp=resp, benchmark_ids=['378b42da']))))
        self.assertEqual(0, len(list(esbench.analyze.benchmarks(resp=resp, benchmark_ids=['foobar']))))
        # this is the response you get when there are no benchmarks
        resp = esbench.api.ApiResponse(status=200, reason='OK', data='{"took":1,"timed_out":false}', curl='curl -XGET http://localhost:9200/stats/bench/_search?sort=benchmark_start:asc&size=100')
        self.assertEqual(0, len(list(esbench.analyze.benchmarks(resp=resp))))


    def test_get_observations(self):
        conn = esbench.api.Conn(conn_cls=esbench.test.test_api.MockHTTPConnection)
        resp = esbench.analyze._get_observations(conn=conn, benchmark_id='foo')
        self.assertEqual(resp.curl, 'curl -XGET http://localhost:9200/%s/obs/_search?q=meta.benchmark_id:foo&sort=meta.observation_start:asc&size=10000' % (esbench.STATS_INDEX_NAME, ))


    def test_observations(self):
#         resp = esbench.analyze._get_observations(esbench.api.Conn(), '315fd1aa')
#         print(resp)
        resp = esbench.api.ApiResponse(status=200, reason='OK', data='{"took":5,"timed_out":false,"_shards":{"total":5,"successful":5,"failed":0},"hits":{"total":10,"max_score":null,"hits":[{"_index":"stats","_type":"obs","_id":"c5ac7b40","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "c5ac7b40", "observation_sequence_no": 1, "observation_start": "2013-11-26T17:58:12Z", "observation_stop": "2013-11-26T17:58:13Z"}, "segments": {"num_committed_segments": 1, "num_search_segments": 1, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}}, "t_optimize": "0.17s", "t_optimize_in_millis": 165}, "stats": {"docs": {"count": 10, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "234ms", "index_time_in_millis": 234, "index_total": 10}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "461ms", "fetch_time_in_millis": 461, "fetch_total": 500, "groups": {"match": {"client_time": "0.34s", "client_time_in_millis": 342, "client_total": 100, "fetch_current": 0, "fetch_time": "113ms", "fetch_time_in_millis": 113, "fetch_total": 100, "query_current": 0, "query_time": "39ms", "query_time_in_millis": 39, "query_total": 100}, "match_abs": {"client_time": "0.29s", "client_time_in_millis": 294, "client_total": 100, "fetch_current": 0, "fetch_time": "86ms", "fetch_time_in_millis": 86, "fetch_total": 100, "query_current": 0, "query_time": "45ms", "query_time_in_millis": 45, "query_total": 100}, "match_no_rnd": {"client_time": "0.45s", "client_time_in_millis": 448, "client_total": 100, "fetch_current": 0, "fetch_time": "156ms", "fetch_time_in_millis": 156, "fetch_total": 100, "query_current": 0, "query_time": "58ms", "query_time_in_millis": 58, "query_total": 100}, "match_srt": {"client_time": "0.32s", "client_time_in_millis": 315, "client_total": 100, "fetch_current": 0, "fetch_time": "102ms", "fetch_time_in_millis": 102, "fetch_total": 100, "query_current": 0, "query_time": "38ms", "query_time_in_millis": 38, "query_total": 100}, "mlt": {"client_time": "0.14s", "client_time_in_millis": 136, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "11ms", "query_time_in_millis": 11, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "193ms", "query_time_in_millis": 193, "query_total": 500}, "store": {"size": "237.5kb", "size_in_bytes": 243242, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488692000]},{"_index":"stats","_type":"obs","_id":"2f74c0e9","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "2f74c0e9", "observation_sequence_no": 2, "observation_start": "2013-11-26T17:58:13Z", "observation_stop": "2013-11-26T17:58:16Z"}, "segments": {"num_committed_segments": 2, "num_search_segments": 2, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}}, "t_optimize": "0.07s", "t_optimize_in_millis": 66}, "stats": {"docs": {"count": 20, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "271ms", "index_time_in_millis": 271, "index_total": 20}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "1.2s", "fetch_time_in_millis": 1206, "fetch_total": 1000, "groups": {"match": {"client_time": "0.84s", "client_time_in_millis": 844, "client_total": 100, "fetch_current": 0, "fetch_time": "189ms", "fetch_time_in_millis": 189, "fetch_total": 100, "query_current": 0, "query_time": "52ms", "query_time_in_millis": 52, "query_total": 100}, "match_abs": {"client_time": "0.35s", "client_time_in_millis": 352, "client_total": 100, "fetch_current": 0, "fetch_time": "135ms", "fetch_time_in_millis": 135, "fetch_total": 100, "query_current": 0, "query_time": "60ms", "query_time_in_millis": 60, "query_total": 100}, "match_no_rnd": {"client_time": "0.57s", "client_time_in_millis": 567, "client_total": 100, "fetch_current": 0, "fetch_time": "230ms", "fetch_time_in_millis": 230, "fetch_total": 100, "query_current": 0, "query_time": "70ms", "query_time_in_millis": 70, "query_total": 100}, "match_srt": {"client_time": "0.46s", "client_time_in_millis": 460, "client_total": 100, "fetch_current": 0, "fetch_time": "188ms", "fetch_time_in_millis": 188, "fetch_total": 100, "query_current": 0, "query_time": "49ms", "query_time_in_millis": 49, "query_total": 100}, "mlt": {"client_time": "0.13s", "client_time_in_millis": 130, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "13ms", "query_time_in_millis": 13, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "439ms", "query_time_in_millis": 439, "query_total": 1000}, "store": {"size": "481.8kb", "size_in_bytes": 493437, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488693000]},{"_index":"stats","_type":"obs","_id":"a00c16c7","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "a00c16c7", "observation_sequence_no": 3, "observation_start": "2013-11-26T17:58:16Z", "observation_stop": "2013-11-26T17:58:18Z"}, "segments": {"num_committed_segments": 3, "num_search_segments": 3, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}, "_2": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 2, "num_docs": 10, "search": true, "size": "374.5kb", "size_in_bytes": 383503, "version": "4.5.1"}}, "t_optimize": "0.05s", "t_optimize_in_millis": 51}, "stats": {"docs": {"count": 30, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "328ms", "index_time_in_millis": 328, "index_total": 30}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "2.1s", "fetch_time_in_millis": 2171, "fetch_total": 1500, "groups": {"match": {"client_time": "0.59s", "client_time_in_millis": 588, "client_total": 100, "fetch_current": 0, "fetch_time": "270ms", "fetch_time_in_millis": 270, "fetch_total": 100, "query_current": 0, "query_time": "51ms", "query_time_in_millis": 51, "query_total": 100}, "match_abs": {"client_time": "0.40s", "client_time_in_millis": 399, "client_total": 100, "fetch_current": 0, "fetch_time": "196ms", "fetch_time_in_millis": 196, "fetch_total": 100, "query_current": 0, "query_time": "54ms", "query_time_in_millis": 54, "query_total": 100}, "match_no_rnd": {"client_time": "0.61s", "client_time_in_millis": 610, "client_total": 100, "fetch_current": 0, "fetch_time": "281ms", "fetch_time_in_millis": 281, "fetch_total": 100, "query_current": 0, "query_time": "60ms", "query_time_in_millis": 60, "query_total": 100}, "match_srt": {"client_time": "0.50s", "client_time_in_millis": 496, "client_total": 100, "fetch_current": 0, "fetch_time": "214ms", "fetch_time_in_millis": 214, "fetch_total": 100, "query_current": 0, "query_time": "50ms", "query_time_in_millis": 50, "query_total": 100}, "mlt": {"client_time": "0.13s", "client_time_in_millis": 133, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "16ms", "query_time_in_millis": 16, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "672ms", "query_time_in_millis": 672, "query_total": 1500}, "store": {"size": "856.4kb", "size_in_bytes": 877007, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488696000]},{"_index":"stats","_type":"obs","_id":"244e5a42","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "244e5a42", "observation_sequence_no": 4, "observation_start": "2013-11-26T17:58:18Z", "observation_stop": "2013-11-26T17:58:21Z"}, "segments": {"num_committed_segments": 5, "num_search_segments": 5, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}, "_2": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 2, "num_docs": 10, "search": true, "size": "374.5kb", "size_in_bytes": 383503, "version": "4.5.1"}, "_3": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 3, "num_docs": 8, "search": true, "size": "305.1kb", "size_in_bytes": 312523, "version": "4.5.1"}, "_4": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 4, "num_docs": 2, "search": true, "size": "132.3kb", "size_in_bytes": 135496, "version": "4.5.1"}}, "t_optimize": "0.02s", "t_optimize_in_millis": 21}, "stats": {"docs": {"count": 40, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "439ms", "index_time_in_millis": 439, "index_total": 40}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "3.1s", "fetch_time_in_millis": 3151, "fetch_total": 2000, "groups": {"match": {"client_time": "0.63s", "client_time_in_millis": 628, "client_total": 100, "fetch_current": 0, "fetch_time": "290ms", "fetch_time_in_millis": 290, "fetch_total": 100, "query_current": 0, "query_time": "56ms", "query_time_in_millis": 56, "query_total": 100}, "match_abs": {"client_time": "0.36s", "client_time_in_millis": 363, "client_total": 100, "fetch_current": 0, "fetch_time": "184ms", "fetch_time_in_millis": 184, "fetch_total": 100, "query_current": 0, "query_time": "49ms", "query_time_in_millis": 49, "query_total": 100}, "match_no_rnd": {"client_time": "0.66s", "client_time_in_millis": 656, "client_total": 100, "fetch_current": 0, "fetch_time": "303ms", "fetch_time_in_millis": 303, "fetch_total": 100, "query_current": 0, "query_time": "57ms", "query_time_in_millis": 57, "query_total": 100}, "match_srt": {"client_time": "0.48s", "client_time_in_millis": 482, "client_total": 100, "fetch_current": 0, "fetch_time": "198ms", "fetch_time_in_millis": 198, "fetch_total": 100, "query_current": 0, "query_time": "61ms", "query_time_in_millis": 61, "query_total": 100}, "mlt": {"client_time": "0.14s", "client_time_in_millis": 137, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "21ms", "query_time_in_millis": 21, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "918ms", "query_time_in_millis": 918, "query_total": 2000}, "store": {"size": "1.2mb", "size_in_bytes": 1325157, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488698000]},{"_index":"stats","_type":"obs","_id":"c0e74e4d","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "c0e74e4d", "observation_sequence_no": 5, "observation_start": "2013-11-26T17:58:21Z", "observation_stop": "2013-11-26T17:58:23Z"}, "segments": {"num_committed_segments": 6, "num_search_segments": 6, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}, "_2": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 2, "num_docs": 10, "search": true, "size": "374.5kb", "size_in_bytes": 383503, "version": "4.5.1"}, "_3": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 3, "num_docs": 8, "search": true, "size": "305.1kb", "size_in_bytes": 312523, "version": "4.5.1"}, "_4": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 4, "num_docs": 2, "search": true, "size": "132.3kb", "size_in_bytes": 135496, "version": "4.5.1"}, "_5": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 5, "num_docs": 10, "search": true, "size": "350kb", "size_in_bytes": 358422, "version": "4.5.1"}}, "t_optimize": "0.04s", "t_optimize_in_millis": 39}, "stats": {"docs": {"count": 50, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "504ms", "index_time_in_millis": 504, "index_total": 50}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "4s", "fetch_time_in_millis": 4067, "fetch_total": 2500, "groups": {"match": {"client_time": "0.60s", "client_time_in_millis": 602, "client_total": 100, "fetch_current": 0, "fetch_time": "266ms", "fetch_time_in_millis": 266, "fetch_total": 100, "query_current": 0, "query_time": "60ms", "query_time_in_millis": 60, "query_total": 100}, "match_abs": {"client_time": "0.35s", "client_time_in_millis": 352, "client_total": 100, "fetch_current": 0, "fetch_time": "175ms", "fetch_time_in_millis": 175, "fetch_total": 100, "query_current": 0, "query_time": "51ms", "query_time_in_millis": 51, "query_total": 100}, "match_no_rnd": {"client_time": "0.60s", "client_time_in_millis": 601, "client_total": 100, "fetch_current": 0, "fetch_time": "267ms", "fetch_time_in_millis": 267, "fetch_total": 100, "query_current": 0, "query_time": "58ms", "query_time_in_millis": 58, "query_total": 100}, "match_srt": {"client_time": "0.49s", "client_time_in_millis": 492, "client_total": 100, "fetch_current": 0, "fetch_time": "205ms", "fetch_time_in_millis": 205, "fetch_total": 100, "query_current": 0, "query_time": "57ms", "query_time_in_millis": 57, "query_total": 100}, "mlt": {"client_time": "0.14s", "client_time_in_millis": 139, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "23ms", "query_time_in_millis": 23, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "1.1s", "query_time_in_millis": 1169, "query_total": 2500}, "store": {"size": "1.6mb", "size_in_bytes": 1683644, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488701000]},{"_index":"stats","_type":"obs","_id":"6ea3adfe","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "6ea3adfe", "observation_sequence_no": 6, "observation_start": "2013-11-26T17:58:23Z", "observation_stop": "2013-11-26T17:58:26Z"}, "segments": {"num_committed_segments": 7, "num_search_segments": 7, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}, "_2": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 2, "num_docs": 10, "search": true, "size": "374.5kb", "size_in_bytes": 383503, "version": "4.5.1"}, "_3": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 3, "num_docs": 8, "search": true, "size": "305.1kb", "size_in_bytes": 312523, "version": "4.5.1"}, "_4": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 4, "num_docs": 2, "search": true, "size": "132.3kb", "size_in_bytes": 135496, "version": "4.5.1"}, "_5": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 5, "num_docs": 10, "search": true, "size": "350kb", "size_in_bytes": 358422, "version": "4.5.1"}, "_6": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 6, "num_docs": 10, "search": true, "size": "388.6kb", "size_in_bytes": 397956, "version": "4.5.1"}}, "t_optimize": "0.03s", "t_optimize_in_millis": 30}, "stats": {"docs": {"count": 60, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "558ms", "index_time_in_millis": 558, "index_total": 60}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "5s", "fetch_time_in_millis": 5092, "fetch_total": 3000, "groups": {"match": {"client_time": "0.68s", "client_time_in_millis": 678, "client_total": 100, "fetch_current": 0, "fetch_time": "307ms", "fetch_time_in_millis": 307, "fetch_total": 100, "query_current": 0, "query_time": "53ms", "query_time_in_millis": 53, "query_total": 100}, "match_abs": {"client_time": "0.38s", "client_time_in_millis": 378, "client_total": 100, "fetch_current": 0, "fetch_time": "205ms", "fetch_time_in_millis": 205, "fetch_total": 100, "query_current": 0, "query_time": "45ms", "query_time_in_millis": 45, "query_total": 100}, "match_no_rnd": {"client_time": "0.68s", "client_time_in_millis": 684, "client_total": 100, "fetch_current": 0, "fetch_time": "311ms", "fetch_time_in_millis": 311, "fetch_total": 100, "query_current": 0, "query_time": "48ms", "query_time_in_millis": 48, "query_total": 100}, "match_srt": {"client_time": "0.48s", "client_time_in_millis": 480, "client_total": 100, "fetch_current": 0, "fetch_time": "198ms", "fetch_time_in_millis": 198, "fetch_total": 100, "query_current": 0, "query_time": "56ms", "query_time_in_millis": 56, "query_total": 100}, "mlt": {"client_time": "0.13s", "client_time_in_millis": 133, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "19ms", "query_time_in_millis": 19, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "1.3s", "query_time_in_millis": 1393, "query_total": 3000}, "store": {"size": "1.9mb", "size_in_bytes": 2081666, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488703000]},{"_index":"stats","_type":"obs","_id":"d7030c18","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "d7030c18", "observation_sequence_no": 7, "observation_start": "2013-11-26T17:58:26Z", "observation_stop": "2013-11-26T17:58:28Z"}, "segments": {"num_committed_segments": 8, "num_search_segments": 8, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}, "_2": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 2, "num_docs": 10, "search": true, "size": "374.5kb", "size_in_bytes": 383503, "version": "4.5.1"}, "_3": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 3, "num_docs": 8, "search": true, "size": "305.1kb", "size_in_bytes": 312523, "version": "4.5.1"}, "_4": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 4, "num_docs": 2, "search": true, "size": "132.3kb", "size_in_bytes": 135496, "version": "4.5.1"}, "_5": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 5, "num_docs": 10, "search": true, "size": "350kb", "size_in_bytes": 358422, "version": "4.5.1"}, "_6": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 6, "num_docs": 10, "search": true, "size": "388.6kb", "size_in_bytes": 397956, "version": "4.5.1"}, "_7": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 7, "num_docs": 10, "search": true, "size": "395.3kb", "size_in_bytes": 404797, "version": "4.5.1"}}, "t_optimize": "0.03s", "t_optimize_in_millis": 34}, "stats": {"docs": {"count": 70, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "634ms", "index_time_in_millis": 634, "index_total": 70}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "6.1s", "fetch_time_in_millis": 6122, "fetch_total": 3500, "groups": {"match": {"client_time": "0.68s", "client_time_in_millis": 683, "client_total": 100, "fetch_current": 0, "fetch_time": "298ms", "fetch_time_in_millis": 298, "fetch_total": 100, "query_current": 0, "query_time": "63ms", "query_time_in_millis": 63, "query_total": 100}, "match_abs": {"client_time": "0.38s", "client_time_in_millis": 378, "client_total": 100, "fetch_current": 0, "fetch_time": "203ms", "fetch_time_in_millis": 203, "fetch_total": 100, "query_current": 0, "query_time": "49ms", "query_time_in_millis": 49, "query_total": 100}, "match_no_rnd": {"client_time": "0.67s", "client_time_in_millis": 673, "client_total": 100, "fetch_current": 0, "fetch_time": "312ms", "fetch_time_in_millis": 312, "fetch_total": 100, "query_current": 0, "query_time": "48ms", "query_time_in_millis": 48, "query_total": 100}, "match_srt": {"client_time": "0.48s", "client_time_in_millis": 476, "client_total": 100, "fetch_current": 0, "fetch_time": "214ms", "fetch_time_in_millis": 214, "fetch_total": 100, "query_current": 0, "query_time": "52ms", "query_time_in_millis": 52, "query_total": 100}, "mlt": {"client_time": "0.13s", "client_time_in_millis": 134, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "20ms", "query_time_in_millis": 20, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "1.6s", "query_time_in_millis": 1627, "query_total": 3500}, "store": {"size": "2.3mb", "size_in_bytes": 2486530, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488706000]},{"_index":"stats","_type":"obs","_id":"8b615e3f","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "8b615e3f", "observation_sequence_no": 8, "observation_start": "2013-11-26T17:58:28Z", "observation_stop": "2013-11-26T17:58:31Z"}, "segments": {"num_committed_segments": 9, "num_search_segments": 9, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}, "_2": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 2, "num_docs": 10, "search": true, "size": "374.5kb", "size_in_bytes": 383503, "version": "4.5.1"}, "_3": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 3, "num_docs": 8, "search": true, "size": "305.1kb", "size_in_bytes": 312523, "version": "4.5.1"}, "_4": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 4, "num_docs": 2, "search": true, "size": "132.3kb", "size_in_bytes": 135496, "version": "4.5.1"}, "_5": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 5, "num_docs": 10, "search": true, "size": "350kb", "size_in_bytes": 358422, "version": "4.5.1"}, "_6": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 6, "num_docs": 10, "search": true, "size": "388.6kb", "size_in_bytes": 397956, "version": "4.5.1"}, "_7": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 7, "num_docs": 10, "search": true, "size": "395.3kb", "size_in_bytes": 404797, "version": "4.5.1"}, "_8": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 8, "num_docs": 10, "search": true, "size": "465.1kb", "size_in_bytes": 476342, "version": "4.5.1"}}, "t_optimize": "0.04s", "t_optimize_in_millis": 35}, "stats": {"docs": {"count": 80, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "693ms", "index_time_in_millis": 693, "index_total": 80}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "7.1s", "fetch_time_in_millis": 7157, "fetch_total": 4000, "groups": {"match": {"client_time": "0.69s", "client_time_in_millis": 691, "client_total": 100, "fetch_current": 0, "fetch_time": "313ms", "fetch_time_in_millis": 313, "fetch_total": 100, "query_current": 0, "query_time": "62ms", "query_time_in_millis": 62, "query_total": 100}, "match_abs": {"client_time": "0.38s", "client_time_in_millis": 379, "client_total": 100, "fetch_current": 0, "fetch_time": "205ms", "fetch_time_in_millis": 205, "fetch_total": 100, "query_current": 0, "query_time": "51ms", "query_time_in_millis": 51, "query_total": 100}, "match_no_rnd": {"client_time": "0.69s", "client_time_in_millis": 687, "client_total": 100, "fetch_current": 0, "fetch_time": "318ms", "fetch_time_in_millis": 318, "fetch_total": 100, "query_current": 0, "query_time": "55ms", "query_time_in_millis": 55, "query_total": 100}, "match_srt": {"client_time": "0.48s", "client_time_in_millis": 483, "client_total": 100, "fetch_current": 0, "fetch_time": "195ms", "fetch_time_in_millis": 195, "fetch_total": 100, "query_current": 0, "query_time": "69ms", "query_time_in_millis": 69, "query_total": 100}, "mlt": {"client_time": "0.13s", "client_time_in_millis": 131, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "22ms", "query_time_in_millis": 22, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "1.8s", "query_time_in_millis": 1889, "query_total": 4000}, "store": {"size": "2.8mb", "size_in_bytes": 2962937, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488708000]},{"_index":"stats","_type":"obs","_id":"a0c28e25","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "a0c28e25", "observation_sequence_no": 9, "observation_start": "2013-11-26T17:58:31Z", "observation_stop": "2013-11-26T17:58:34Z"}, "segments": {"num_committed_segments": 1, "num_search_segments": 1, "segments": {"_a": {"committed": true, "compound": false, "deleted_docs": 0, "generation": 10, "num_docs": 90, "search": true, "size": "2.7mb", "size_in_bytes": 2853865, "version": "4.5.1"}}, "t_optimize": "0.46s", "t_optimize_in_millis": 457}, "stats": {"docs": {"count": 90, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "730ms", "index_time_in_millis": 730, "index_total": 90}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 1, "total_docs": 90, "total_size": "3mb", "total_size_in_bytes": 3248531, "total_time": "423ms", "total_time_in_millis": 423}, "search": {"fetch_current": 0, "fetch_time": "8.3s", "fetch_time_in_millis": 8334, "fetch_total": 4500, "groups": {"match": {"client_time": "0.65s", "client_time_in_millis": 654, "client_total": 100, "fetch_current": 0, "fetch_time": "326ms", "fetch_time_in_millis": 326, "fetch_total": 100, "query_current": 0, "query_time": "25ms", "query_time_in_millis": 25, "query_total": 100}, "match_abs": {"client_time": "0.35s", "client_time_in_millis": 351, "client_total": 100, "fetch_current": 0, "fetch_time": "210ms", "fetch_time_in_millis": 210, "fetch_total": 100, "query_current": 0, "query_time": "18ms", "query_time_in_millis": 18, "query_total": 100}, "match_no_rnd": {"client_time": "0.67s", "client_time_in_millis": 668, "client_total": 100, "fetch_current": 0, "fetch_time": "334ms", "fetch_time_in_millis": 334, "fetch_total": 100, "query_current": 0, "query_time": "22ms", "query_time_in_millis": 22, "query_total": 100}, "match_srt": {"client_time": "0.65s", "client_time_in_millis": 646, "client_total": 100, "fetch_current": 0, "fetch_time": "304ms", "fetch_time_in_millis": 304, "fetch_total": 100, "query_current": 0, "query_time": "28ms", "query_time_in_millis": 28, "query_total": 100}, "mlt": {"client_time": "0.11s", "client_time_in_millis": 108, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "7ms", "query_time_in_millis": 7, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "1.9s", "query_time_in_millis": 1993, "query_total": 4500}, "store": {"size": "2.7mb", "size_in_bytes": 2854630, "throttle_time": "120.3ms", "throttle_time_in_millis": 120}}},"sort":[1385488711000]},{"_index":"stats","_type":"obs","_id":"6d183b4c","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "6d183b4c", "observation_sequence_no": 10, "observation_start": "2013-11-26T17:58:34Z", "observation_stop": "2013-11-26T17:58:36Z"}, "segments": {"num_committed_segments": 2, "num_search_segments": 2, "segments": {"_a": {"committed": true, "compound": false, "deleted_docs": 0, "generation": 10, "num_docs": 90, "search": true, "size": "2.7mb", "size_in_bytes": 2853865, "version": "4.5.1"}, "_b": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 11, "num_docs": 10, "search": true, "size": "231.2kb", "size_in_bytes": 236834, "version": "4.5.1"}}, "t_optimize": "0.03s", "t_optimize_in_millis": 26}, "stats": {"docs": {"count": 100, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "765ms", "index_time_in_millis": 765, "index_total": 100}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 1, "total_docs": 90, "total_size": "3mb", "total_size_in_bytes": 3248531, "total_time": "423ms", "total_time_in_millis": 423}, "search": {"fetch_current": 0, "fetch_time": "9.4s", "fetch_time_in_millis": 9449, "fetch_total": 5000, "groups": {"match": {"client_time": "0.63s", "client_time_in_millis": 627, "client_total": 100, "fetch_current": 0, "fetch_time": "302ms", "fetch_time_in_millis": 302, "fetch_total": 100, "query_current": 0, "query_time": "26ms", "query_time_in_millis": 26, "query_total": 100}, "match_abs": {"client_time": "0.34s", "client_time_in_millis": 342, "client_total": 100, "fetch_current": 0, "fetch_time": "195ms", "fetch_time_in_millis": 195, "fetch_total": 100, "query_current": 0, "query_time": "26ms", "query_time_in_millis": 26, "query_total": 100}, "match_no_rnd": {"client_time": "0.64s", "client_time_in_millis": 637, "client_total": 100, "fetch_current": 0, "fetch_time": "300ms", "fetch_time_in_millis": 300, "fetch_total": 100, "query_current": 0, "query_time": "35ms", "query_time_in_millis": 35, "query_total": 100}, "match_srt": {"client_time": "0.66s", "client_time_in_millis": 662, "client_total": 100, "fetch_current": 0, "fetch_time": "315ms", "fetch_time_in_millis": 315, "fetch_total": 100, "query_current": 0, "query_time": "29ms", "query_time_in_millis": 29, "query_total": 100}, "mlt": {"client_time": "0.12s", "client_time_in_millis": 122, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "10ms", "query_time_in_millis": 10, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "2.1s", "query_time_in_millis": 2122, "query_total": 5000}, "store": {"size": "2.9mb", "size_in_bytes": 3091110, "throttle_time": "120.3ms", "throttle_time_in_millis": 120}}},"sort":[1385488714000]}]}}', curl='curl -XGET http://localhost:9200/stats/obs/_search?q=meta.benchmark_id:315fd1aa&sort=meta.observation_start:asc&size=10000')
        obs = list(esbench.analyze.observations(resp=resp))
        self.assertEqual(10, len(obs))


#     def test_stat_tuple(self):
#         OBSERVATION = '[{"sort": [1385146907000], "_type": "bench", "_source": {"benchmark_id": "378b42da", "cmnd": "run --name t1 --repetitions 500 --observations 20 500000", "t_total_in_millis": 7632910, "benchmark_start": "2013-11-22T19:01:47Z", "argv": {"no_optimize_calls": false, "verbose": false, "name": "t1", "segments": null, "repetitions": 500, "n": 500000, "no_load": false, "command": "run", "observations": 20, "data": null, "append": false, "config_file_path": "/Users/z013rqg/dev/esbench/esbench/config.json"}, "t_total": "127.22m", "benchmark_stop": "2013-11-22T21:09:00Z", "config": "{\\"index\\": {\\"mappings\\": {\\"doc\\": {\\"_size\\": {\\"enabled\\": true, \\"store\\": \\"yes\\"}, \\"_source\\": {\\"enabled\\": true}, \\"properties\\": {\\"abstract\\": {\\"properties\\": {\\"txt\\": {\\"store\\": \\"yes\\", \\"type\\": \\"string\\"}}}}}}, \\"settings\\": {\\"index\\": {\\"number_of_replicas\\": 0, \\"number_of_shards\\": 1}}}, \\"name_doctype\\": \\"doc\\", \\"name_index\\": \\"test\\", \\"queries\\": {\\"match\\": {\\"fields\\": [\\"description.txt\\"], \\"from\\": 0, \\"query\\": {\\"match\\": {\\"description.txt\\": \\"computing V%(variable)s device portable\\"}}, \\"size\\": 10}, \\"match_abs\\": {\\"fields\\": [\\"abstract.txt\\"], \\"from\\": 0, \\"query\\": {\\"match\\": {\\"description.txt\\": \\"computing V%(variable)s device portable\\"}}, \\"size\\": 10}, \\"match_no_rnd\\": {\\"fields\\": [\\"description.txt\\"], \\"from\\": 0, \\"query\\": {\\"match\\": {\\"description.txt\\": \\"computing device portable\\"}}, \\"size\\": 10}, \\"match_srt\\": {\\"fields\\": [\\"description.txt\\"], \\"from\\": 0, \\"query\\": {\\"match\\": {\\"description.txt\\": \\"computing V%(variable)s device portable\\"}}, \\"size\\": 10, \\"sort\\": [{\\"_meta.date_published\\": \\"desc\\"}]}, \\"mlt\\": {\\"fields\\": [\\"description.txt\\"], \\"from\\": 0, \\"query\\": {\\"more_like_this\\": {\\"fields\\": [\\"description.txt\\"], \\"like_text\\": \\"USING V%(variable)s PROTECTING THE CARTRIDGE FROM WELDING HEAT\\", \\"max_query_terms\\": 25, \\"min_term_freq\\": 2, \\"percent_terms_to_match\\": 0.3}}, \\"size\\": 10}}}", "benchmark_name": "t1"}, "_score": null, "_index": "stats", "_id": "378b42da"}, {"sort": [1385147123000], "_type": "obs", "_source": {"segments": {"segments": {"_4z": {"search": true, "num_docs": 212, "deleted_docs": 0, "generation": 179, "size_in_bytes": 6726696, "committed": true, "version": "4.5.1", "compound": true, "size": "6.4mb"}, "_58": {"search": true, "num_docs": 150, "deleted_docs": 0, "generation": 188, "size_in_bytes": 7115018, "committed": true, "version": "4.5.1", "compound": true, "size": "6.7mb"}, "_4r": {"search": true, "num_docs": 21, "deleted_docs": 0, "generation": 171, "size_in_bytes": 3892488, "committed": true, "version": "4.5.1", "compound": true, "size": "3.7mb"}, "_4s": {"search": true, "num_docs": 8, "deleted_docs": 0, "generation": 172, "size_in_bytes": 1211823, "committed": true, "version": "4.5.1", "compound": true, "size": "1.1mb"}, "_4t": {"search": true, "num_docs": 35, "deleted_docs": 0, "generation": 173, "size_in_bytes": 4655082, "committed": true, "version": "4.5.1", "compound": true, "size": "4.4mb"}, "_56": {"search": true, "num_docs": 134, "deleted_docs": 0, "generation": 186, "size_in_bytes": 7316354, "committed": true, "version": "4.5.1", "compound": true, "size": "6.9mb"}, "_4w": {"search": true, "num_docs": 1403, "deleted_docs": 0, "generation": 176, "size_in_bytes": 70211226, "committed": true, "version": "4.5.1", "compound": false, "size": "66.9mb"}, "_4h": {"search": true, "num_docs": 186, "deleted_docs": 0, "generation": 161, "size_in_bytes": 6360394, "committed": true, "version": "4.5.1", "compound": true, "size": "6mb"}, "_4m": {"search": true, "num_docs": 1877, "deleted_docs": 0, "generation": 166, "size_in_bytes": 64564042, "committed": true, "version": "4.5.1", "compound": false, "size": "61.5mb"}, "_4c": {"search": true, "num_docs": 1419, "deleted_docs": 0, "generation": 156, "size_in_bytes": 72852182, "committed": true, "version": "4.5.1", "compound": false, "size": "69.4mb"}, "_x": {"search": true, "num_docs": 7, "deleted_docs": 0, "generation": 33, "size_in_bytes": 471943, "committed": true, "version": "4.5.1", "compound": true, "size": "460.8kb"}, "_3q": {"search": true, "num_docs": 1338, "deleted_docs": 0, "generation": 134, "size_in_bytes": 78297105, "committed": true, "version": "4.5.1", "compound": false, "size": "74.6mb"}, "_3s": {"search": true, "num_docs": 26, "deleted_docs": 0, "generation": 136, "size_in_bytes": 4377327, "committed": true, "version": "4.5.1", "compound": true, "size": "4.1mb"}, "_3u": {"search": true, "num_docs": 21, "deleted_docs": 0, "generation": 138, "size_in_bytes": 3503868, "committed": true, "version": "4.5.1", "compound": true, "size": "3.3mb"}, "_3t": {"search": true, "num_docs": 8, "deleted_docs": 0, "generation": 137, "size_in_bytes": 1412926, "committed": true, "version": "4.5.1", "compound": true, "size": "1.3mb"}, "_41": {"search": true, "num_docs": 14766, "deleted_docs": 0, "generation": 145, "size_in_bytes": 664812107, "committed": true, "version": "4.5.1", "compound": false, "size": "634mb"}, "_59": {"search": true, "num_docs": 152, "deleted_docs": 0, "generation": 189, "size_in_bytes": 7063709, "committed": true, "version": "4.5.1", "compound": true, "size": "6.7mb"}, "_5c": {"search": true, "num_docs": 121, "deleted_docs": 0, "generation": 192, "size_in_bytes": 5391921, "committed": true, "version": "4.5.1", "compound": true, "size": "5.1mb"}, "_5b": {"search": true, "num_docs": 151, "deleted_docs": 0, "generation": 191, "size_in_bytes": 6752394, "committed": true, "version": "4.5.1", "compound": true, "size": "6.4mb"}, "_5a": {"search": true, "num_docs": 157, "deleted_docs": 0, "generation": 190, "size_in_bytes": 6854586, "committed": true, "version": "4.5.1", "compound": true, "size": "6.5mb"}, "_57": {"search": true, "num_docs": 1129, "deleted_docs": 0, "generation": 187, "size_in_bytes": 79916427, "committed": true, "version": "4.5.1", "compound": false, "size": "76.2mb"}, "_2u": {"search": true, "num_docs": 13, "deleted_docs": 0, "generation": 102, "size_in_bytes": 579296, "committed": true, "version": "4.5.1", "compound": true, "size": "565.7kb"}, "_36": {"search": true, "num_docs": 1666, "deleted_docs": 0, "generation": 114, "size_in_bytes": 59023716, "committed": true, "version": "4.5.1", "compound": false, "size": "56.2mb"}}, "num_search_segments": 23, "t_optimize": "17.77s", "t_optimize_in_millis": 17770, "num_committed_segments": 23}, "meta": {"observation_id": "32415fcd", "benchmark_id": "378b42da", "observation_stop": "2013-11-22T19:05:36Z", "observation_start": "2013-11-22T19:05:23Z", "observation_sequence_no": 1}, "stats": {"docs": {"count": 25000, "deleted": 0}, "search": {"query_total": 2500, "query_time": "4.9s", "fetch_current": 0, "fetch_total": 2500, "fetch_time": "3.8s", "query_time_in_millis": 4901, "open_contexts": 0, "fetch_time_in_millis": 3889, "groups": {"match_no_rnd": {"query_total": 500, "query_time": "1.1s", "fetch_current": 0, "fetch_total": 500, "fetch_time": "1s", "query_time_in_millis": 1108, "client_time_in_millis": 3365, "client_total": 500, "fetch_time_in_millis": 1082, "query_current": 0, "client_time": "3.37s"}, "match_abs": {"query_total": 500, "query_time": "1.1s", "fetch_current": 0, "fetch_total": 500, "fetch_time": "653ms", "query_time_in_millis": 1118, "client_time_in_millis": 2305, "client_total": 500, "fetch_time_in_millis": 653, "query_current": 0, "client_time": "2.31s"}, "match_srt": {"query_total": 500, "query_time": "1.3s", "fetch_current": 0, "fetch_total": 500, "fetch_time": "1s", "query_time_in_millis": 1328, "client_time_in_millis": 3560, "client_total": 500, "fetch_time_in_millis": 1062, "query_current": 0, "client_time": "3.56s"}, "match": {"query_total": 500, "query_time": "1.1s", "fetch_current": 0, "fetch_total": 500, "fetch_time": "1s", "query_time_in_millis": 1163, "client_time_in_millis": 3420, "client_total": 500, "fetch_time_in_millis": 1087, "query_current": 0, "client_time": "3.42s"}, "mlt": {"query_total": 500, "query_time": "182ms", "fetch_current": 0, "fetch_total": 500, "fetch_time": "2ms", "query_time_in_millis": 182, "client_time_in_millis": 605, "client_total": 500, "fetch_time_in_millis": 2, "query_current": 0, "client_time": "0.61s"}}, "query_current": 0}, "indexing": {"delete_time_in_millis": 0, "index_total": 25000, "index_current": 0, "delete_time": "0s", "delete_total": 0, "index_time": "2.5m", "index_time_in_millis": 152779, "delete_current": 0}, "store": {"size_in_bytes": 1163365664, "throttle_time_in_millis": 136856, "throttle_time": "2.2m", "size": "1gb"}, "merges": {"total_time": "2.8m", "total_docs": 38364, "total_size_in_bytes": 1796069413, "total_size": "1.6gb", "current_size": "0b", "current_size_in_bytes": 0, "current_docs": 0, "current": 0, "total_time_in_millis": 171955, "total": 17}}}, "_score": null, "_index": "stats", "_id": "32415fcd"}, ["match_no_rnd", {"query_total": 500, "query_time": "1.1s", "fetch_current": 0, "fetch_total": 500, "fetch_time": "1s", "query_time_in_millis": 1108, "client_time_in_millis": 3365, "client_total": 500, "fetch_time_in_millis": 1082, "query_current": 0, "client_time": "3.37s"}]]'
#         benchmark, observation, stat = json.loads(OBSERVATION)
#         t = esbench.analyze.stat_tuple(benchmark, observation, stat)
#         self.assertEqual(t, esbench.analyze.StatRecord(bench_id=u'378b42da', bench_name=u't1', obs_id=u'32415fcd', obs_no=1, doc_cnt=25000, seg_cnt=23, size_b=1163365664, t_index_ms=152779, query_name=u'match_no_rnd', n_query=500, t_query_ms=1108, n_fetch=500, t_fetch_ms=1082, n_client=500, t_client_ms=3365))



if __name__ == "__main__":
    unittest.main()

